@@ -52,6 +52,7 @@ struct _EMailNotesEditor {
 	EFocusTracker *focus_tracker;
 	GtkActionGroup *action_group;
 	GBinding *attachment_paned_binding;
+	EMenuBar *menu_bar;
 
 	gboolean had_message;
 	CamelMimeMessage *message;
@@ -1023,6 +1024,7 @@ e_mail_notes_editor_dispose (GObject *object)
 	g_clear_object (&notes_editor->focus_tracker);
 	g_clear_object (&notes_editor->action_group);
 	g_clear_object (&notes_editor->attachment_paned_binding);
+	g_clear_object (&notes_editor->menu_bar);
 
 	/* Chain up to parent's method */
 	G_OBJECT_CLASS (e_mail_notes_editor_parent_class)->dispose (object);
@@ -1129,9 +1131,10 @@ e_mail_notes_editor_new_with_editor (EHTMLEditor *html_editor,
 	EFocusTracker *focus_tracker;
 	EActivityBar *activity_bar;
 	GtkUIManager *ui_manager;
-	GtkWidget *widget, *content;
+	GtkWidget *widget, *content, *button, *menu_button = NULL;
 	GtkActionGroup *action_group;
 	GtkAction *action;
+	GtkHeaderBar *header_bar;
 	GSettings *settings;
 	GError *local_error = NULL;
 
@@ -1141,7 +1144,6 @@ e_mail_notes_editor_new_with_editor (EHTMLEditor *html_editor,
 		"transient-for", parent,
 		"destroy-with-parent", TRUE,
 		"window-position", GTK_WIN_POS_CENTER_ON_PARENT,
-		"title", _("Edit Message Note"),
 		NULL);
 
 	gtk_window_set_default_size (GTK_WINDOW (notes_editor), 600, 440);
@@ -1181,12 +1183,40 @@ e_mail_notes_editor_new_with_editor (EHTMLEditor *html_editor,
 	/* Construct the window content. */
 
 	widget = e_html_editor_get_managed_widget (notes_editor->editor, "/main-menu");
+	notes_editor->menu_bar = e_menu_bar_new (GTK_MENU_BAR (widget), GTK_WINDOW (notes_editor), &menu_button);
 	gtk_box_pack_start (GTK_BOX (content), widget, FALSE, FALSE, 0);
-	gtk_widget_show (widget);
 
-	widget = e_html_editor_get_managed_widget (notes_editor->editor, "/main-toolbar");
-	gtk_box_pack_start (GTK_BOX (content), widget, FALSE, FALSE, 0);
-	gtk_widget_show (widget);
+	if (e_util_get_use_header_bar ()) {
+		widget = gtk_header_bar_new ();
+		gtk_widget_show (widget);
+		header_bar = GTK_HEADER_BAR (widget);
+		gtk_header_bar_set_show_close_button (header_bar, TRUE);
+		gtk_header_bar_set_title (header_bar, _("Edit Message Note"));
+		gtk_window_set_titlebar (GTK_WINDOW (notes_editor), widget);
+
+		action = gtk_action_group_get_action (notes_editor->action_group, "save-and-close");
+		button = e_header_bar_button_new (_("Save"), action);
+		e_header_bar_button_css_add_class (E_HEADER_BAR_BUTTON (button), "suggested-action");
+		gtk_widget_show (button);
+		gtk_header_bar_pack_start (header_bar, button);
+
+		widget = e_html_editor_get_managed_widget (notes_editor->editor, "/main-toolbar/pre-main-toolbar/save-and-close");
+		gtk_widget_destroy (widget);
+
+		if (menu_button)
+			gtk_header_bar_pack_end (header_bar, menu_button);
+	} else {
+		gtk_window_set_title (GTK_WINDOW (notes_editor), _("Edit Message Note"));
+
+		widget = e_html_editor_get_managed_widget (notes_editor->editor, "/main-toolbar");
+		gtk_box_pack_start (GTK_BOX (content), widget, FALSE, FALSE, 0);
+		gtk_widget_show (widget);
+
+		if (menu_button) {
+			g_object_ref_sink (menu_button);
+			gtk_widget_destroy (menu_button);
+		}
+	}
 
 	widget = GTK_WIDGET (notes_editor->editor);
 	g_object_set (G_OBJECT (widget),
